----------------------------------------------------------------------------------------------------
-- MAASLAND AXE SCRIPT
----------------------------------------------------------------------------------------------------
-- Purpose:  Creates a new handtool, type maaslandAxe, for axes
-- Authors:  Rahkiin
--
-- For: Maaslandmods
--
-- Copyright (c) Jos Kuijpers, Timmiej93 2018
----------------------------------------------------------------------------------------------------

MaaslandAxe = {}
MaaslandAxe_mt = Class(MaaslandAxe, HandTool)

InitObjectClass(MaaslandAxe, "MaaslandAxe")

-- getfenv(0)["MaaslandAxe"] = MaaslandAxe

function MaaslandAxe:new(isServer, isClient, customMt)
    local mt = customMt
    if mt == nil then
        mt = MaaslandAxe_mt
    end

    local self = HandTool:new(isServer, isClient, mt)
    -- registerObjectClassName(self, "MaaslandAxe")

    return self
end

function MaaslandAxe:load(xmlFilename, player)
    if not MaaslandAxe:superClass().load(self, xmlFilename, player) then
        return false
    end

    local xmlFile = loadXMLFile("TempXML", xmlFilename)

    self.rotationZ = 0
    self.rotationSpeedZ = 0.003
    self.cutSizeY = 1.1
    self.cutSizeZ = 1
    self.isCutting = false
    self.waitingForResetAfterCut = false
    self.cutNode = getChildAt(self.rootNode, 0)
    self.graphicsNode = getChildAt(self.cutNode, 0)
    self.chainNode = getChildAt(self.graphicsNode, 0)
    self.psNode = getChildAt(self.graphicsNode, 1)
    self.doDelimb = false
    self.hasMovedY = false

    self.startSwing = MaaslandAxe.startSwing

    self.pricePerSecond = Utils.getNoNil(getXMLFloat(xmlFile, "handTool.axe.pricePerMinute"), 50) / 1000

    if self.isClient then
        self.particleSystems = {}

        local i = 0
        while true do
            local keyPS = string.format("handTool.axe.particleSystems.emitterShape(%d)", i)
            if not hasXMLProperty(xmlFile, keyPS) then
                break
            end
            local emitterShape = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, keyPS.."#node"))
            local particleType = getXMLString(xmlFile, keyPS.."#particleType")
            if emitterShape ~= nil then
                local fillType = FillUtil.FILLTYPE_WOODCHIPS
                local particleSystem = MaterialUtil.getParticleSystem(fillType, particleType)
                if particleSystem ~= nil then
                    table.insert(self.particleSystems, ParticleUtil.copyParticleSystem(xmlFile, keyPS, particleSystem, emitterShape))
                end
            end
            i = i + 1
        end

        if #self.particleSystems == 0 then
            self.particleSystems = nil
        end

        self.handNode = Utils.getNoNil(Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "handTool.axe.handNode#index")), self.rootNode)
        self.handNodeRotation = Utils.getRadiansFromString(Utils.getNoNil(getXMLString(xmlFile, "handTool.axe.handNode#rotation"), "0 0 0"), 3)

        self.equipmentUVs = Utils.getVectorNFromString(Utils.getNoNil(getXMLString(xmlFile, "handTool.axe.equipment#uvs"), "0 0"), 2)

        self.chain = Utils.loadScrollers(self.rootNode, xmlFile, "handTool.axe.chain", {}, false)
        table.getn(self.chain)

        self.sampleWork = SoundUtil.loadSample(xmlFile, {}, "handTool.axe.workSound", nil, self.baseDirectory)

        self.samplesBranch = {}
        local i=0
        while true do
            if not hasXMLProperty(xmlFile, string.format("handTool.axe.branchSounds.branchSound(%d)#file",i)) then
                break
            end
            local sampleBranch = SoundUtil.loadSample(xmlFile, {}, string.format("handTool.axe.branchSounds.branchSound(%d)",i), nil, self.baseDirectory)
            table.insert(self.samplesBranch, sampleBranch)
            i = i + 1
        end
        self.samplesBranchCount = i
        self.samplesBranchActiveTimer = 0

        local filename = getXMLString(xmlFile, "handTool.axe.ringSelector#file")
        if filename ~= nil then
            local i3dNode = Utils.loadSharedI3DFile(filename, self.baseDirectory, false, false, false)
            if i3dNode ~= 0 then
                self.ringSelectorFilename = filename
                self.ringSelector = getChildAt(i3dNode, 0)
                self.ringSelectorScaleOffset = Utils.getNoNil(getXMLFloat(xmlFile, "handTool.axe.ringSelector#scaleOffset"), 0.3)
                setVisibility(self.ringSelector, false)
                link(self.cutNode, self.ringSelector)
                delete(i3dNode)
            end
        end
    end

    self.needGloves = true
    self.needHelmet = false
    self.lastWorkTime = 0
    self.maxWorkTime = 300

    self.speedFactor = 0

    self.offsetY = 0
    self.moveSpeedY = 0.00006
    self.speedFactor = 0

    self.quickTapDuration = 0
    self.quickTapDurationTime = 250 -- mouse/button presses shorter than this will play the quick tap sound
    self.lastQuickTapTime = 0
    self.quickTapMinDelay = 250 -- quick tap sound won't play again until after this amount of time

    self.workPlaying = false

    self.isCutting = false
    self.isHorizontalCut = false

    self.currentHandNode = nil

    self.swingCounter = 0
    self.soundPlayed = false
    self.updateSwing = MaaslandAxe.updateSwing

    delete(xmlFile)

    return true
end

function MaaslandAxe:delete()
    MaaslandAxe:superClass().delete(self)
    if self.isClient then
        ParticleUtil.deleteParticleSystems(self.particleSystems)

        SoundUtil.deleteSample(self.sampleWork)

        for _,v in pairs(self.samplesBranch) do
            SoundUtil.deleteSample(v)
        end
        if self.ringSelectorFilename ~= nil then
            Utils.releaseSharedI3DFile(self.ringSelectorFilename, self.baseDirectory, true)
        end
    end
end

function MaaslandAxe:update(dt, allowInput)
    MaaslandAxe:superClass().update(self, dt, allowInput)

    self:updateSwing(dt)

    Utils.updateScrollers(self.chain, dt*self.speedFactor, true)

    if self.isServer then
        local price = self.pricePerSecond * (dt / 1000)
        g_currentMission.missionStats:updateStats("expenses", price)
        g_currentMission:addSharedMoney(-price, "vehicleRunningCost")
    end

    self.shouldDelimb = false

    local lockPlayerInput = false

    if allowInput then
        local isCutting = false

        -- if not SoundUtil.isSamplePlaying(self.sampleStart, 1.5*dt) then
        --     SoundUtil.playSample(self.sampleWork, 0, 0, 0)
        -- end

        -- setRotation(self.graphicsNode, math.rad(math.random(-1, 1))*0.1, math.rad(math.random(-1, 1))*0.1, math.rad(-180))

        if self.curSplitShape == nil then
            local input = InputBinding.getDigitalInputAxis(InputBinding.AXIS_ROTATE_HANDTOOL)
            if InputBinding.isAxisZero(input) then
                input = InputBinding.getAnalogInputAxis(InputBinding.AXIS_ROTATE_HANDTOOL)
            end
            lockPlayerInput = input ~= 0

            if input ~= 0 then
                self.rotationZ = Utils.clamp(self.rotationZ + self.rotationSpeedZ*input*dt, -1.57, 1.57)
                setRotation(self.rootNode, 0, 0, self.rotationZ)
                self.player:setIKDirty()
            end

            -- self.offsetY = math.max(self.offsetY - 5*self.moveSpeedY*dt, 0)
            -- setTranslation(self.graphicsNode, 0, self.offsetY, 0)
            -- if self.offsetY ~= 0 then
            --     self.player:setIKDirty()
            -- end
        end

        local shape = 0
        if not self.waitingForResetAfterCut then
            local x,y,z = getWorldTranslation(self.cutNode)
            local nx,ny,nz = localDirectionToWorld(self.cutNode, 1,0,0)
            local yx,yy,yz = localDirectionToWorld(self.cutNode, 0,1,0)
            if self.curSplitShape ~= nil or self.offsetY == 0 then
                local minY,maxY, minZ,maxZ
                if self.curSplitShape == nil or not entityExists(self.curSplitShape) then
                    self.curSplitShape = nil
                    shape, minY,maxY, minZ,maxZ = findSplitShape(x,y,z, nx,ny,nz, yx,yy,yz, self.cutSizeY, self.cutSizeZ)

                    if shape ~= nil and shape ~= 0 then
                        local cutTooLow = false
                        local _,y1,_ = localToLocal(self.cutNode, shape, 0,minY,minZ)
                        local _,y3,_ = localToLocal(self.cutNode, shape, 0,maxY,minZ)
                        local _,y4,_ = localToLocal(self.cutNode, shape, 0,maxY,maxZ)
                        cutTooLow = y1 < 0.01 or y1 < 0.01 or y3 < 0.03 or y4 < 0.01
                        if not cutTooLow then
                            local x1,y1,z1 = localToWorld(self.cutNode, 0,minY,minZ)
                            local x2,y2,z2 = localToWorld(self.cutNode, 0,minY,maxZ)
                            local x3,y3,z3 = localToWorld(self.cutNode, 0,maxY,minZ)
                            local x4,y4,z4 = localToWorld(self.cutNode, 0,maxY,maxZ)
                            local h1 = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x1,y1,z1)
                            local h2 = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x2,y2,z2)
                            local h3 = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x3,y3,z3)
                            local h4 = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x4,y4,z4)
                            cutTooLow = h1 < 0.01 or h2 < 0.01 or h3 < 0.03 or h4 < 0.01
                        end
                        if cutTooLow then
                            self.player.walkingIsLocked = false
                            self.curSplitShape = nil
                            shape, minY,maxY, minZ,maxZ = 0, nil,nil, nil,nil
                        end
                    end
                end

                self.curSplitShapeMinY = minY
                self.curSplitShapeMaxY = maxY
                self.curSplitShapeMinZ = minZ
                self.curSplitShapeMaxZ = maxZ
            end
        end

        if self.ringSelector ~= nil then
            setVisibility(self.ringSelector, (self.curSplitShapeMinY ~= nil or self.curSplitShape ~= nil) and g_woodCuttingMarkerEnabled)
            if g_woodCuttingMarkerEnabled then
                self:updateRingSelector()
            end
        end

    	self.doAnimation = InputBinding.isPressed(InputBinding.ACTIVATE_HANDTOOL)
        if InputBinding.isPressed(InputBinding.ACTIVATE_HANDTOOL) then

            self.quickTapDuration = self.quickTapDuration + dt

            self.speedFactor = math.min(self.speedFactor + dt/self.maxWorkTime, 1)

            if self.quickTapDuration > self.quickTapDurationTime then
                self.lastWorkTime = math.min(self.lastWorkTime+dt, self.maxWorkTime)

                -- if not self.workPlaying then
                --     SoundUtil.setSampleVolume(self.sampleWork, 1)
                --     self.workPlaying = true
                -- end
            end

            if not self.waitingForResetAfterCut then
                self.shouldDelimb = true

                local x,y,z = getWorldTranslation(self.cutNode)
                local nx,ny,nz = localDirectionToWorld(self.cutNode, 1,0,0)
                local yx,yy,yz = localDirectionToWorld(self.cutNode, 0,1,0)

                if self.curSplitShape ~= nil or self.offsetY == 0 then
                    if self.curSplitShape ~= nil and entityExists(self.curSplitShape) then
                        lockPlayerInput = true
                        local minY,maxY, minZ,maxZ = testSplitShape(self.curSplitShape, x,y,z, nx,ny,nz, yx,yy,yz, self.cutSizeY, self.cutSizeZ)
                        if minY == nil then
                            -- cancel cutting if shape can't be cut anymore from current position
                            self.player.walkingIsLocked = false
                            self.curSplitShape = nil
                            -- SoundUtil.setSamplePitch(self.sampleWork, 1)
                        else
                            local cutTooLow = false
                            local _,y1,_ = localToLocal(self.cutNode, self.curSplitShape, 0,minY,minZ)
                            local _,y3,_ = localToLocal(self.cutNode, self.curSplitShape, 0,maxY,minZ)
                            local _,y4,_ = localToLocal(self.cutNode, self.curSplitShape, 0,maxY,maxZ)
                            cutTooLow = y1 < 0.01 or y1 < 0.01 or y3 < 0.03 or y4 < 0.01
                            if not cutTooLow then
                                local x1,y1,z1 = localToWorld(self.cutNode, 0,minY,minZ)
                                local x2,y2,z2 = localToWorld(self.cutNode, 0,minY,maxZ)
                                local x3,y3,z3 = localToWorld(self.cutNode, 0,maxY,minZ)
                                local x4,y4,z4 = localToWorld(self.cutNode, 0,maxY,maxZ)
                                local h1 = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x1,y1,z1)
                                local h2 = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x2,y2,z2)
                                local h3 = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x3,y3,z3)
                                local h4 = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x4,y4,z4)
                                cutTooLow = h1 < 0.01 or h2 < 0.01 or h3 < 0.03 or h4 < 0.01
                            end
                            if cutTooLow then
                                self.player.walkingIsLocked = false
                                self.curSplitShape = nil
                            end
                        end
                        self.curSplitShapeMinY = minY
                        self.curSplitShapeMaxY = maxY
                        self.curSplitShapeMinZ = minZ
                        self.curSplitShapeMaxZ = maxZ
                    else
                        if shape ~= 0 then
                            self.player.walkingIsLocked = true
                            self.curSplitShape = shape
                        end
                    end

                    self:updateRingSelector()

                    if self.curSplitShape ~= nil then
                        isCutting = true
                        -- self.offsetY = self.offsetY + self.moveSpeedY*dt
                        -- setTranslation(self.graphicsNode, 0, self.offsetY, 0)
                        -- self.player:setIKDirty()

                        if self.offsetY > self.curSplitShapeMinY then
                            self.lastWorkTime = math.min(self.lastWorkTime, self.maxWorkTime*0.7)
                            -- SoundUtil.setSamplePitch(self.sampleWork, 0.96)
                        end

                        if self.offsetY > self.curSplitShapeMaxY then
                            if g_currentMission:getIsServer() then
                                ChainsawUtil.cutSplitShape(self.curSplitShape, x,y,z, nx,ny,nz, yx,yy,yz, self.cutSizeY, self.cutSizeZ)
                            else
                                g_client:getServerConnection():sendEvent(ChainsawCutEvent:new(self.curSplitShape, x,y,z, nx,ny,nz, yx,yy,yz, self.cutSizeY, self.cutSizeZ))
                            end
                            self.player.walkingIsLocked = false
                            self.waitingForResetAfterCut = true
                            self.curSplitShape = nil
                            self.curSplitShapeMinY = nil
                            -- SoundUtil.setSamplePitch(self.sampleWork, 1)
                        end
                    end
                end
            end
        else
            self.speedFactor = math.max(self.speedFactor - dt/self.maxWorkTime, 0)
            self.waitingForResetAfterCut = false
            self.player.walkingIsLocked = false
            self.curSplitShape = nil
            self.curSplitShapeMinY = nil
            self.lastWorkTime = math.max(self.lastWorkTime - dt, 0)

            if self.quickTapDuration > 0 then
                if self.quickTapDuration < self.quickTapDurationTime then
                else
                    -- SoundUtil.setSampleVolume(self.sampleWork, 0)
                    -- self.workPlaying = false
                    -- SoundUtil.setSamplePitch(self.sampleWork, 1)
                end
            end

            self.quickTapDuration = 0
        end

        self.player:lockInput(lockPlayerInput)

        -- if self.particleSystems ~= nil then
        --     local active = false
        --     if (self.samplesBranchActiveTimer > g_currentMission.time) or (self.curSplitShapeMinY ~= nil and  self.curSplitShapeMaxY ~= nil and self.offsetY > self.curSplitShapeMinY and self.offsetY < self.curSplitShapeMaxY) then
        --         active = true
        --     end
        --     for _, ps in pairs(self.particleSystems) do
        --         ParticleUtil.setEmittingState(ps, active)
        --     end
        -- end

        self:setCutting(isCutting, self.rotationZ > 0.7)
    end
end

function MaaslandAxe:updateRingSelector()
    if self.curSplitShape ~= nil then
        setShaderParameter(self.ringSelector, "colorScale", 0.395, 0.925, 0.115, 1, false)
    else
        setShaderParameter(self.ringSelector, "colorScale", 0.098, 0.450, 0.960, 1, false)
    end

    if self.curSplitShapeMinY ~= nil then
        local a,b,c = localToWorld(self.cutNode, 0, (self.curSplitShapeMinY+self.curSplitShapeMaxY)*0.5, (self.curSplitShapeMinZ+self.curSplitShapeMaxZ)*0.5)
        local x, y, z = worldToLocal(getParent(self.ringSelector), a,b,c)
        setTranslation(self.ringSelector, x,y,z)
        local scale = math.max(self.curSplitShapeMaxY-self.curSplitShapeMinY + self.ringSelectorScaleOffset, self.curSplitShapeMaxZ-self.curSplitShapeMinZ + self.ringSelectorScaleOffset)
        setScale(self.ringSelector, 1, scale, scale)
    end
end

function MaaslandAxe:updateTick(dt, allowInput)
    MaaslandAxe:superClass().updateTick(self, dt, allowInput)
    -- if self.isClient then
    --     if self.shouldDelimb then
    --         local x,y,z = getWorldTranslation(self.cutNode)
    --         local nx,ny,nz = localDirectionToWorld(self.cutNode, 1,0,0)
    --         local yx,yy,yz = localDirectionToWorld(self.cutNode, 0,1,0)
    --         if g_server == nil then
    --             -- TODO: g_client:getServerConnection():sendEvent(MaaslandAxeDelimbEvent:new(self.player, x,y,z, nx,ny,nz, yx,yy,yz, false))
    --         else
    --             local ret = findAndRemoveSplitShapeAttachments(x,y,z, nx,ny,nz, yx,yy,yz, 0.7, self.cutSizeY, self.cutSizeZ)
    --             if ret then
    --                 self:setOnDelimb(true)
    --             end
    --         end
    --     end
    -- end
end

function MaaslandAxe:setCutting(isCutting, isHorizontalCut, noEventSend)
    -- TODO MaaslandAxeStateEvent.sendEvent(self.player, isCutting, isHorizontalCut, noEventSend)
    self.isCutting = isCutting
    self.isHorizontalCut = isHorizontalCut
    if g_currentMission.player ~= self.player then
        self.player:setCuttingAnim(isCutting, isHorizontalCut)
    end
end

function MaaslandAxe:setOnDelimb(state)
    if state == true then
        if not (self.samplesBranchActiveTimer > g_currentMission.time) then
            local idx = math.floor(math.random(1, self.samplesBranchCount))
            SoundUtil.playSample(self.samplesBranch[idx], 1, 0, nil)
            self.samplesBranchActiveTimer = g_currentMission.time + self.samplesBranch[idx].duration

            self.lastWorkTime = math.max(0, self.lastWorkTime - 160) --10*dt)
        end
    end
end

function MaaslandAxe:draw()
    MaaslandAxe:superClass().draw(self)
    g_currentMission:addHelpButtonText(g_i18n:getText("input_ACTIVATE_HANDTOOL"), InputBinding.ACTIVATE_HANDTOOL)
    g_currentMission:addHelpButtonText(g_i18n:getText("action_rotate"), InputBinding.AXIS_ROTATE_HANDTOOL)
end

function MaaslandAxe:setHandNode(handNode)
    MaaslandAxe:superClass().setHandNode(self, handNode)
    if self.currentHandNode ~= handNode then
        if g_currentMission.player ~= self.player then
            link(handNode, self.rootNode)
            self.currentHandNode = handNode
            setRotation(self.rootNode, unpack(self.handNodeRotation))
            local x,y,z = getWorldTranslation(self.handNode)
            x,y,z = worldToLocal(getParent(self.rootNode), x,y,z)
            local a,b,c = getTranslation(self.rootNode)
            setTranslation(self.rootNode, a-x,b-y,c-z)
        end
    end
end

function MaaslandAxe:onActivate(allowInput)
    MaaslandAxe:superClass().onActivate(self)
    self.player:setEquipmentUVs(self.equipmentUVs)
    if self.isClient and allowInput then
        -- SoundUtil.playSample(self.sampleWork, 0, 0, 0)
    end
end

function MaaslandAxe:onDeactivate(allowInput)
    MaaslandAxe:superClass().onDeactivate(self)
    self.speedFactor = 0
    self.curSplitShape = nil
    self.player.walkingIsLocked = false

    if self.isClient then
        self.offsetY = 0
        setTranslation(self.graphicsNode, 0, 0, 0)
        -- if self.particleSystems ~= nil then
        --     for _, ps in pairs(self.particleSystems) do
        --         ParticleUtil.resetNumOfEmittedParticles(ps)
        --         ParticleUtil.setEmittingState(ps, false)
        --     end
        -- end

        -- SoundUtil.stopSample(self.sampleWork, true)
    end
end

function MaaslandAxe:startSwing(noEventSend)

    self.swingCounter = 1
    AxeSwingEvent.sendEvent(self.player, true, self.rotationZ, noEventSend)
end

function MaaslandAxe:updateSwing(dt)

	if (not (self.waitingForResetAfterCut and self.swingCounter <= 0)) then
		if (self.doAnimation or self.swingCounter > 0) then
            if (self.swingCounter <= 0) then
                self:startSwing()
            end
			self.swingCounter = self.swingCounter + dt

			local rotBack = -55
			local normCounter = (self.swingCounter / 500)
			if (normCounter <= 1) then
				local rotNorm = (math.cos((math.pi*normCounter) + math.pi) + 1) / 2

				local _,y,z = getRotation(self.graphicsNode)
				setRotation(self.graphicsNode, (math.rad(normCounter*rotBack)),y,z)
			end

			local rotForw = 90
			normCounter = ((self.swingCounter-500) / 200)
			if (normCounter <= 1 and normCounter > 0) then
				local rotNorm = (math.cos((0.5*math.pi*normCounter) + math.pi) + 1)

				-- Degrees of rotation from rotBack
				local localRotation = (math.abs(rotBack)+math.abs(rotForw))*rotNorm

				local _,y,z = getRotation(self.graphicsNode)
				setRotation(self.graphicsNode, (math.rad(localRotation+rotBack)),y,z)
			end

            local active = getVisibility(self.ringSelector)
			if self.particleSystems ~= nil then

	            for _, ps in pairs(self.particleSystems) do
		            if (active and self.swingCounter >= 680 and self.swingCounter <= 750) then
		            	ParticleUtil.setEmittingState(ps, true)
		            else
		            	ParticleUtil.setEmittingState(ps, false)
		                ParticleUtil.resetNumOfEmittedParticles(ps)
			        end
		        end
	        end

            if (self.isClient and self.swingCounter >= 680 and self.swingCounter < 750) then
                if self.shouldDelimb then
                    local x,y,z = getWorldTranslation(self.cutNode)
                    local nx,ny,nz = localDirectionToWorld(self.cutNode, 1,0,0)
                    local yx,yy,yz = localDirectionToWorld(self.cutNode, 0,1,0)
                    if g_server == nil then
                        g_client:getServerConnection():sendEvent(ChainsawDelimbEvent:new(self.player, x,y,z, nx,ny,nz, yx,yy,yz, false))
                    else
                        local ret = findAndRemoveSplitShapeAttachments(x,y,z, nx,ny,nz, yx,yy,yz, 0.7, self.cutSizeY, self.cutSizeZ)
                        if ret then
                            self:setOnDelimb(true)

                            if (not self.doDelimb) then
                                self.doDelimb = true
                            end
                        end
                    end
                end
            end

            if (self.curSplitShape ~= nil and not self.hasMovedY and self.swingCounter >= 700) then
                self.offsetY = self.offsetY + self.moveSpeedY*1000
                setTranslation(self.graphicsNode, 0, self.offsetY, 0)
                self.player:setIKDirty()
                self.hasMovedY = true
            end

            if ((active or self.doDelimb) and self.swingCounter >= 680 and self.swingCounter <= 750) then
                if (not self.soundPlayed) then
                    self.soundPlayed = true
                    if (self.doDelimb) then
                    	SoundUtil.setSamplePitch(self.sampleWork, 1.5)
                    else
                    	SoundUtil.setSamplePitch(self.sampleWork, 1.0)
                    end
                	SoundUtil.playSample(self.sampleWork, 1, 0, 50)
                end
            end

			normCounter = ((self.swingCounter-700) / 300)
			if (normCounter <= 1 and normCounter > 0) then
				local rotNorm = (math.cos((math.pi*normCounter) + math.pi) + 1) / 2

				local _,y,z = getRotation(self.graphicsNode)
				setRotation(self.graphicsNode, (math.rad(rotForw-(rotForw*normCounter))),y,z)
			end

			if (normCounter > 1) then
				self.swingCounter = -1
                self.soundPlayed = false
                self.doDelimb = false
                self.hasMovedY = false
                if (not self.doAnimation) then
                    AxeSwingEvent.sendEvent(self.player, false, 0)
                end
			end

	        self.player:setIKDirty()
	    end
	end

    if (not self.doAnimation) then

    	self.offsetY = math.max(self.offsetY - 10*self.moveSpeedY*dt, 0)
        setTranslation(self.graphicsNode, 0, self.offsetY, 0)

        self.player:setIKDirty()
    end
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ --
-- Ask permission event
--   Send this event to all administrators when non-admin wants to repair a tire

AxeSwingEvent = {}
AxeSwingEvent_mt = Class(AxeSwingEvent, Event)

InitEventClass(AxeSwingEvent, "AxeSwingEvent")

function AxeSwingEvent:emptyNew()
    local self = Event:new(AxeSwingEvent_mt)
    return self
end

function AxeSwingEvent:new(player, doSwing, rotation)
    local self = AxeSwingEvent:emptyNew()
    self.player = player
    self.doSwing = doSwing
    self.rotation = rotation
    return self
end

function AxeSwingEvent:readStream(streamId, connection)
    self.player = readNetworkNodeObject(streamId)
    self.doSwing = streamReadBool(streamId)
    self.rotation = streamReadInt8(streamId)
    self:run(connection)
end

function AxeSwingEvent:writeStream(streamId, connection)
    writeNetworkNodeObject(streamId, self.player)
    streamWriteBool(streamId, self.doSwing)
    streamWriteInt8(streamId, self.rotation)
end

function AxeSwingEvent:run(connection)
    if (not connection:getIsServer()) then
        g_server:broadcastEvent(self, false, connection, self.object)
    end

    if (self.player ~= g_currentMission.player) then

	    local currentTool = self.player.currentTool
	    if (currentTool ~= nil and currentTool.swingCounter ~= nil) then
	        setRotation(currentTool.rootNode, 0, 0, self.rotation)
	        if (self.doSwing) then
	            currentTool:startSwing(true)
	        end
    	end
    end
end

function AxeSwingEvent.sendEvent(player, doSwing, rotation, noEventSend)
    if (noEventSend == nil or noEventSend == false) then
        if (g_server ~= nil) then
            g_server:broadcastEvent(AxeSwingEvent:new(player, doSwing, rotation), nil, nil, player)
        else
            g_client:getServerConnection():sendEvent(AxeSwingEvent:new(player, doSwing, rotation))
        end
    end
end

registerHandTool("maaslandAxe", MaaslandAxe)
